@@ -26,9 +26,7 @@ want_syslog=0
 syslog_tag=
 user='@MYSQLD_USER@'
 pid_file=
-pid_file_append=
 err_log=
-err_log_append=
 
 syslog_tag_mysqld=mysqld
 syslog_tag_mysqld_safe=mysqld_safe
@@ -603,17 +601,15 @@ then
       err_log="$err_log".err
     fi
 
-    err_log_append="$err_log"
     case "$err_log" in
       /* ) ;;
       * ) err_log="$DATADIR/$err_log" ;;
     esac
   else
     err_log=$DATADIR/`@HOSTNAME@`.err
-    err_log_append=`@HOSTNAME@`.err
   fi
 
-  append_arg_to_args "--log-error=$err_log_append"
+  append_arg_to_args "--log-error=$err_log"
 
   if [ $want_syslog -eq 1 ]
   then
@@ -728,15 +724,13 @@ fi
 if test -z "$pid_file"
 then
   pid_file="$DATADIR/`@HOSTNAME@`.pid"
-  pid_file_append="`@HOSTNAME@`.pid"
 else
-  pid_file_append="$pid_file"
   case "$pid_file" in
     /* ) ;;
     * )  pid_file="$DATADIR/$pid_file" ;;
   esac
 fi
-append_arg_to_args "--pid-file=$pid_file_append"
+append_arg_to_args "--pid-file=$pid_file"
 
 if test -n "$mysql_unix_port"
 then
