@@ -3326,6 +3326,8 @@ logs_empty_and_mark_files_at_shutdown(void)
 	algorithm only works if the server is idle at shutdown */
 
 	srv_shutdown_state = SRV_SHUTDOWN_CLEANUP;
+
+	srv_wake_purge_thread();
 loop:
 	os_thread_sleep(100000);
 
@@ -3499,7 +3501,7 @@ loop:
 		srv_shutdown_state = SRV_SHUTDOWN_LAST_PHASE;
 		/* Wake the log tracking thread which will then immediatelly
 		quit because of srv_shutdown_state value */
-		if (srv_track_changed_pages) {
+		if (srv_redo_log_thread_started) {
 			os_event_set(srv_checkpoint_completed_event);
 			os_event_wait(srv_redo_log_thread_finished_event);
 		}
@@ -3576,7 +3578,7 @@ loop:
 	srv_shutdown_state = SRV_SHUTDOWN_LAST_PHASE;
 
 	/* Signal the log following thread to quit */
-	if (srv_track_changed_pages) {
+	if (srv_redo_log_thread_started) {
 		os_event_set(srv_checkpoint_completed_event);
 	}
 
@@ -3600,7 +3602,7 @@ loop:
 
 	fil_flush_file_spaces(FIL_TABLESPACE);
 
-	if (srv_track_changed_pages) {
+	if (srv_redo_log_thread_started) {
 		os_event_wait(srv_redo_log_thread_finished_event);
 	}
 
