@@ -767,7 +767,7 @@ ulonglong mi_safe_mul(ulonglong a, ulonglong b)
 
 	/* Set up functions in structs */
 
-void mi_setup_functions(register MYISAM_SHARE *share)
+void mi_setup_functions(MYISAM_SHARE *share)
 {
   if (share->options & HA_OPTION_COMPRESS_RECORD)
   {
@@ -821,7 +821,7 @@ void mi_setup_functions(register MYISAM_SHARE *share)
 }
 
 
-static void setup_key_functions(register MI_KEYDEF *keyinfo)
+static void setup_key_functions(MI_KEYDEF *keyinfo)
 {
   if (keyinfo->key_alg == HA_KEY_ALG_RTREE)
   {
